@@ -12,8 +12,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.Free
  */
 
 /* By Owen Taylor <otaylor@gtk.org>              98/4/4 */
@@ -50,25 +49,23 @@
  * Together with #GtkSocket, #GtkPlug provides the ability to embed
  * widgets from one process into another process in a fashion that is
  * transparent to the user. One process creates a #GtkSocket widget
- * and passes the ID of that widget's window to the other process,
+ * and passes the ID of that widget’s window to the other process,
  * which then creates a #GtkPlug with that window ID. Any widgets
  * contained in the #GtkPlug then will appear inside the first
- * application's window.
+ * application’s window.
  *
  * The communication between a #GtkSocket and a #GtkPlug follows the
- * <ulink url="http://www.freedesktop.org/Standards/xembed-spec">XEmbed</ulink>
- * protocol. This protocol has also been implemented in other toolkits,
- * e.g. <application>Qt</application>, allowing the same level of
- * integration when embedding a <application>Qt</application> widget
+ * [XEmbed Protocol](http://www.freedesktop.org/Standards/xembed-spec).
+ * This protocol has also been implemented in other toolkits,
+ * e.g. Qt, allowing the same level of
+ * integration when embedding a Qt widget
  * in GTK+ or vice versa.
  *
- * <note>
  * The #GtkPlug and #GtkSocket widgets are only available when GTK+
  * is compiled for the X11 platform and %GDK_WINDOWING_X11 is defined.
  * They can only be used on a #GdkX11Display. To use #GtkPlug and
- * #GtkSocket, you need to include the <filename>gtk/gtkx.h</filename>
+ * #GtkSocket, you need to include the `gtk/gtkx.h`
  * header.
- * </note>
  */
 
 struct _GtkPlugPrivate
@@ -131,7 +128,7 @@ enum {
 
 static guint plug_signals[LAST_SIGNAL] = { 0 };
 
-G_DEFINE_TYPE (GtkPlug, gtk_plug, GTK_TYPE_WINDOW)
+G_DEFINE_TYPE_WITH_PRIVATE (GtkPlug, gtk_plug, GTK_TYPE_WINDOW)
 
 static void
 gtk_plug_get_property (GObject    *object,
@@ -234,21 +231,16 @@ gtk_plug_class_init (GtkPlugClass *class)
 		  NULL, NULL,
 		  _gtk_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
-
-  g_type_class_add_private (class, sizeof (GtkPlugPrivate));
 }
 
 static void
 gtk_plug_init (GtkPlug *plug)
 {
-  plug->priv = G_TYPE_INSTANCE_GET_PRIVATE (plug,
-                                            GTK_TYPE_PLUG,
-                                            GtkPlugPrivate);
+  plug->priv = gtk_plug_get_instance_private (plug);
 }
 
 /**
  * gtk_plug_handle_modality_on:
- *
  * @plug: a #GtkPlug
  *
  * Called from the GtkPlug backend when the corresponding socket has
@@ -272,7 +264,6 @@ gtk_plug_handle_modality_on (GtkPlug *plug)
 
 /**
  * gtk_plug_handle_modality_off:
- *
  * @plug: a #GtkPlug
  *
  * Called from the GtkPlug backend when the corresponding socket has
@@ -320,7 +311,9 @@ gtk_plug_set_is_child (GtkPlug  *plug,
 	gtk_widget_unmap (widget);
 
       _gtk_window_set_is_toplevel (GTK_WINDOW (plug), FALSE);
+      G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
       gtk_container_set_resize_mode (GTK_CONTAINER (plug), GTK_RESIZE_PARENT);
+      G_GNUC_END_IGNORE_DEPRECATIONS;
 
       _gtk_widget_propagate_hierarchy_changed (widget, widget);
     }
@@ -335,7 +328,9 @@ gtk_plug_set_is_child (GtkPlug  *plug,
       gtk_window_group_add_window (priv->modality_group, GTK_WINDOW (plug));
 
       _gtk_window_set_is_toplevel (GTK_WINDOW (plug), TRUE);
+      G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
       gtk_container_set_resize_mode (GTK_CONTAINER (plug), GTK_RESIZE_QUEUE);
+      G_GNUC_END_IGNORE_DEPRECATIONS;
 
       _gtk_widget_propagate_hierarchy_changed (GTK_WIDGET (plug), NULL);
     }
@@ -349,7 +344,7 @@ gtk_plug_set_is_child (GtkPlug  *plug,
  * be used to embed this window inside another window, for
  * instance with gtk_socket_add_id().
  * 
- * Return value: the window ID for the plug
+ * Returns: the window ID for the plug
  **/
 Window
 gtk_plug_get_id (GtkPlug *plug)
@@ -368,7 +363,7 @@ gtk_plug_get_id (GtkPlug *plug)
  *
  * Determines whether the plug is embedded in a socket.
  *
- * Return value: %TRUE if the plug is embedded in a socket
+ * Returns: %TRUE if the plug is embedded in a socket
  *
  * Since: 2.14
  **/
@@ -386,7 +381,7 @@ gtk_plug_get_embedded (GtkPlug *plug)
  *
  * Retrieves the socket the plug is embedded in.
  *
- * Return value: (transfer none): the window of the socket, or %NULL
+ * Returns: (transfer none): the window of the socket, or %NULL
  *
  * Since: 2.14
  **/
@@ -444,16 +439,16 @@ _gtk_plug_add_to_socket (GtkPlug   *plug,
   g_signal_emit_by_name (socket_, "plug-added");
 }
 
-/**
- * _gtk_plug_send_delete_event:
+/*
+ * gtk_plug_send_delete_event:
  * @widget: a #GtkWidget
  *
  * Send a GDK_DELETE event to the @widget and destroy it if
  * necessary. Internal GTK function, called from this file or the
  * backend-specific GtkPlug implementation.
  */
-void
-_gtk_plug_send_delete_event (GtkWidget *widget)
+static void
+gtk_plug_send_delete_event (GtkWidget *widget)
 {
   GdkEvent *event = gdk_event_new (GDK_DELETE);
 
@@ -484,6 +479,7 @@ _gtk_plug_remove_from_socket (GtkPlug   *plug,
   GtkPlugPrivate *priv;
   GtkWidget *widget;
   GdkWindow *window;
+  GdkWindow *root_window;
   gboolean result;
   gboolean widget_was_visible;
 
@@ -502,12 +498,11 @@ _gtk_plug_remove_from_socket (GtkPlug   *plug,
 
   widget_was_visible = gtk_widget_get_visible (widget);
   window = gtk_widget_get_window (widget);
+  root_window = gdk_screen_get_root_window (gtk_widget_get_screen (widget));
 
   gdk_window_hide (window);
   _gtk_widget_set_in_reparent (widget, TRUE);
-  gdk_window_reparent (window,
-		       gtk_widget_get_root_window (widget),
-		       0, 0);
+  gdk_window_reparent (window, root_window, 0, 0);
   gtk_widget_unparent (GTK_WIDGET (plug));
   _gtk_widget_set_in_reparent (widget, FALSE);
   
@@ -533,7 +528,7 @@ _gtk_plug_remove_from_socket (GtkPlug   *plug,
     gtk_widget_destroy (GTK_WIDGET (socket_));
 
   if (window)
-    _gtk_plug_send_delete_event (widget);
+    gtk_plug_send_delete_event (widget);
 
   g_object_unref (plug);
 
@@ -546,7 +541,7 @@ _gtk_plug_remove_from_socket (GtkPlug   *plug,
 /**
  * gtk_plug_construct:
  * @plug: a #GtkPlug.
- * @socket_id: the XID of the socket's window.
+ * @socket_id: the XID of the socket’s window.
  *
  * Finish the initialization of @plug for a given #GtkSocket identified by
  * @socket_id. This function will generally only be used by classes deriving from #GtkPlug.
@@ -561,9 +556,9 @@ gtk_plug_construct (GtkPlug *plug,
 /**
  * gtk_plug_construct_for_display:
  * @plug: a #GtkPlug.
- * @display: the #GdkDisplay associated with @socket_id's 
+ * @display: the #GdkDisplay associated with @socket_id’s 
  *	     #GtkSocket.
- * @socket_id: the XID of the socket's window.
+ * @socket_id: the XID of the socket’s window.
  *
  * Finish the initialization of @plug for a given #GtkSocket identified by
  * @socket_id which is currently displayed on @display.
@@ -625,10 +620,10 @@ gtk_plug_construct_for_display (GtkPlug    *plug,
  * @socket_id:  the window ID of the socket, or 0.
  * 
  * Creates a new plug widget inside the #GtkSocket identified
- * by @socket_id. If @socket_id is 0, the plug is left "unplugged" and
+ * by @socket_id. If @socket_id is 0, the plug is left “unplugged” and
  * can later be plugged into a #GtkSocket by  gtk_socket_add_id().
  * 
- * Return value: the new #GtkPlug widget.
+ * Returns: the new #GtkPlug widget.
  **/
 GtkWidget*
 gtk_plug_new (Window socket_id)
@@ -639,11 +634,11 @@ gtk_plug_new (Window socket_id)
 /**
  * gtk_plug_new_for_display:
  * @display: the #GdkDisplay on which @socket_id is displayed
- * @socket_id: the XID of the socket's window.
+ * @socket_id: the XID of the socket’s window.
  * 
  * Create a new plug widget inside the #GtkSocket identified by socket_id.
  *
- * Return value: the new #GtkPlug widget.
+ * Returns: the new #GtkPlug widget.
  *
  * Since: 2.2
  */
@@ -680,7 +675,6 @@ gtk_plug_unrealize (GtkWidget *widget)
 
   if (priv->socket_window != NULL)
     {
-      gdk_window_set_user_data (priv->socket_window, NULL);
       g_object_unref (priv->socket_window);
       priv->socket_window = NULL;
 
@@ -720,7 +714,6 @@ xembed_set_info (GdkWindow     *window,
 
 /**
  * gtk_plug_focus_first_last:
- *
  * @plug: a #GtkPlug
  * @direction: a direction
  *
@@ -883,7 +876,6 @@ gtk_plug_filter_func (GdkXEvent *gdk_xevent,
 	      {
 		GtkWidget *widget = GTK_WIDGET (plug);
 
-                gdk_window_set_user_data (priv->socket_window, NULL);
 		g_object_unref (priv->socket_window);
 		priv->socket_window = NULL;
 
@@ -899,7 +891,7 @@ gtk_plug_filter_func (GdkXEvent *gdk_xevent,
 		if (xre->parent == GDK_WINDOW_XID (gdk_screen_get_root_window (screen)))
 		  {
 		    GTK_NOTE (PLUGSOCKET, g_message ("GtkPlug: calling gtk_plug_send_delete_event()"));
-		    _gtk_plug_send_delete_event (widget);
+		    gtk_plug_send_delete_event (widget);
 
 		    g_object_notify (G_OBJECT (plug), "embedded");
 		  }
@@ -986,6 +978,10 @@ gtk_plug_filter_func (GdkXEvent *gdk_xevent,
         gdk_event_set_device (event, keyboard);
 
         keymap = gdk_keymap_get_for_display (display);
+
+        event->key.group = gdk_x11_keymap_get_group_for_state (keymap, xevent->xkey.state);
+        event->key.is_modifier = gdk_x11_keymap_key_is_modifier (keymap, event->key.hardware_keycode);
+
         gdk_keymap_translate_keyboard_state (keymap,
                                              event->key.hardware_keycode,
                                              event->key.state,
@@ -1000,10 +996,6 @@ gtk_plug_filter_func (GdkXEvent *gdk_xevent,
         event->key.length = 0;
         event->key.string = g_strdup ("");
 
-        /* FIXME: These should be filled in properly */
-        event->key.group = 0;
-        event->key.is_modifier = FALSE;
-
         return_val = GDK_FILTER_TRANSLATE;
       }
     }
@@ -1054,15 +1046,25 @@ gtk_plug_realize (GtkWidget *widget)
 
   if (gtk_widget_is_toplevel (widget))
     {
+      GdkWindow *root_window;
       attributes.window_type = GDK_WINDOW_TOPLEVEL;
 
+      root_window = gdk_screen_get_root_window (gtk_widget_get_screen (widget));
+
       gdk_error_trap_push ();
       if (priv->socket_window)
         gdk_window = gdk_window_new (priv->socket_window,
                                      &attributes, attributes_mask);
       else /* If it's a passive plug, we use the root window */
-        gdk_window = gdk_window_new (gtk_widget_get_root_window (widget),
+        gdk_window = gdk_window_new (root_window,
                                      &attributes, attributes_mask);
+      /* Because the window isn't known to the window manager,
+       * frame sync won't work. In theory, XEMBED could be extended
+       * so that embedder did frame sync like a window manager, but
+       * it's just not worth the effort considering the current
+       * minimal use of XEMBED.
+       */
+      gdk_x11_window_set_frame_sync_enabled (gdk_window, FALSE);
       gtk_widget_set_window (widget, gdk_window);
 
       gdk_display_sync (gtk_widget_get_display (widget));
@@ -1071,8 +1073,8 @@ gtk_plug_realize (GtkWidget *widget)
 	  gdk_error_trap_push ();
 	  gdk_window_destroy (gdk_window);
 	  gdk_error_trap_pop_ignored ();
-	  gdk_window = gdk_window_new (gtk_widget_get_root_window (widget),
-                                   &attributes, attributes_mask);
+	  gdk_window = gdk_window_new (root_window,
+                                       &attributes, attributes_mask);
           gtk_widget_set_window (widget, gdk_window);
 	}
 
@@ -1092,12 +1094,10 @@ gtk_plug_realize (GtkWidget *widget)
       gtk_widget_set_window (widget, gdk_window);
     }
 
-  gdk_window_set_user_data (gdk_window, window);
+  gtk_widget_register_window (widget, gdk_window);
 
   gtk_style_context_set_background (gtk_widget_get_style_context (widget),
                                     gdk_window);
-
-  gdk_window_enable_synchronized_configure (gdk_window);
 }
 
 static void
