@@ -66,13 +66,6 @@
  * never need to use most of the functions in this section directly;
  * #GtkClipboard provides a nicer interface to the same functionality.
  *
- * If an application is expected to exchange image data and work
- * on Windows, it is highly advised to support at least "image/bmp" target
- * for the widest possible compatibility with third-party applications.
- * #GtkClipboard already does that by using gtk_target_list_add_image_targets()
- * and gtk_selection_data_set_pixbuf() or gtk_selection_data_get_pixbuf(),
- * which is one of the reasons why it is advised to use #GtkClipboard.
- *
  * Some of the datatypes defined this section are used in
  * the #GtkClipboard and drag-and-drop API’s as well. The
  * #GtkTargetEntry and #GtkTargetList objects represent
@@ -82,9 +75,6 @@
  * associated information.
  */
 
-/* We are using deprecated API, here, and we know that */
-#define GDK_DISABLE_DEPRECATION_WARNINGS
-
 #include "config.h"
 
 #include "gtkselection.h"
@@ -108,26 +98,16 @@
 #include "win32/gdkwin32.h"
 #endif
 
-#ifdef GDK_WINDOWING_WAYLAND
-#include <gdk/wayland/gdkwayland.h>
-#endif
-
-#ifdef GDK_WINDOWING_BROADWAY
-#include "broadway/gdkbroadway.h"
-#endif
-
 #undef DEBUG_SELECTION
 
 /* Maximum size of a sent chunk, in bytes. Also the default size of
    our buffers */
 #ifdef GDK_WINDOWING_X11
 #define GTK_SELECTION_MAX_SIZE(display)                                 \
-  GDK_IS_X11_DISPLAY (display) ?                                        \
   MIN(262144,                                                           \
       XExtendedMaxRequestSize (GDK_DISPLAY_XDISPLAY (display)) == 0     \
        ? XMaxRequestSize (GDK_DISPLAY_XDISPLAY (display)) - 100         \
-       : XExtendedMaxRequestSize (GDK_DISPLAY_XDISPLAY (display)) - 100)\
-  : G_MAXINT
+       : XExtendedMaxRequestSize (GDK_DISPLAY_XDISPLAY (display)) - 100)
 #else
 /* No chunks on Win32 */
 #define GTK_SELECTION_MAX_SIZE(display) G_MAXINT
@@ -234,8 +214,7 @@ static const char gtk_selection_handler_key[] = "gtk-selection-handlers";
 
 /**
  * gtk_target_list_new:
- * @targets: (array length=ntargets) (allow-none): Pointer to an array
- *   of #GtkTargetEntry
+ * @targets: (array length=ntargets): Pointer to an array of #GtkTargetEntry
  * @ntargets: number of entries in @targets.
  * 
  * Creates a new #GtkTargetList from an array of #GtkTargetEntry.
@@ -642,7 +621,9 @@ gtk_target_table_new_from_list (GtkTargetList *list,
   *n_targets = g_list_length (list->list);
   targets = g_new0 (GtkTargetEntry, *n_targets);
 
-  for (tmp_list = list->list, i = 0; tmp_list; tmp_list = tmp_list->next, i++)
+  for (i = 0, tmp_list = list->list;
+       i < *n_targets;
+       i++, tmp_list = g_list_next (tmp_list))
     {
       GtkTargetPair *pair = tmp_list->data;
 
@@ -895,15 +876,6 @@ gtk_selection_clear_targets (GtkWidget *widget,
   g_return_if_fail (GTK_IS_WIDGET (widget));
   g_return_if_fail (selection != GDK_NONE);
 
-#ifdef GDK_WINDOWING_WAYLAND
-  if (GDK_IS_WAYLAND_DISPLAY (gtk_widget_get_display (widget)))
-    gdk_wayland_selection_clear_targets (gtk_widget_get_display (widget), selection);
-#endif
-#ifdef GDK_WINDOWING_WIN32
-  if (GDK_IS_WIN32_DISPLAY (gtk_widget_get_display (widget)))
-    gdk_win32_selection_clear_targets (gtk_widget_get_display (widget), selection);
-#endif
-
   lists = g_object_get_data (G_OBJECT (widget), gtk_selection_handler_key);
   
   tmp_list = lists;
@@ -948,13 +920,8 @@ gtk_selection_add_target (GtkWidget	    *widget,
 
   list = gtk_selection_target_list_get (widget, selection);
   gtk_target_list_add (list, target, 0, info);
-#ifdef GDK_WINDOWING_WAYLAND
-  if (GDK_IS_WAYLAND_DISPLAY (gtk_widget_get_display (widget)))
-    gdk_wayland_selection_add_targets (gtk_widget_get_window (widget), selection, 1, &target);
-#endif
 #ifdef GDK_WINDOWING_WIN32
-  if (GDK_IS_WIN32_DISPLAY (gtk_widget_get_display (widget)))
-    gdk_win32_selection_add_targets (gtk_widget_get_window (widget), selection, 1, &target);
+  gdk_win32_selection_add_targets (gtk_widget_get_window (widget), selection, 1, &target);
 #endif
 }
 
@@ -983,31 +950,16 @@ gtk_selection_add_targets (GtkWidget            *widget,
   list = gtk_selection_target_list_get (widget, selection);
   gtk_target_list_add_table (list, targets, ntargets);
 
-#ifdef GDK_WINDOWING_WAYLAND
-  if (GDK_IS_WAYLAND_DISPLAY (gtk_widget_get_display (widget)))
-    {
-      GdkAtom *atoms = g_new (GdkAtom, ntargets);
-      guint i;
-
-      for (i = 0; i < ntargets; i++)
-        atoms[i] = gdk_atom_intern (targets[i].target, FALSE);
-
-      gdk_wayland_selection_add_targets (gtk_widget_get_window (widget), selection, ntargets, atoms);
-      g_free (atoms);
-    }
-#endif
-
 #ifdef GDK_WINDOWING_WIN32
-  if (GDK_IS_WIN32_DISPLAY (gtk_widget_get_display (widget)))
-    {
-      int i;
-      GdkAtom *atoms = g_new (GdkAtom, ntargets);
+  {
+    int i;
+    GdkAtom *atoms = g_new (GdkAtom, ntargets);
 
-      for (i = 0; i < ntargets; ++i)
-        atoms[i] = gdk_atom_intern (targets[i].target, FALSE);
-      gdk_win32_selection_add_targets (gtk_widget_get_window (widget), selection, ntargets, atoms);
-      g_free (atoms);
-    }
+    for (i = 0; i < ntargets; ++i)
+      atoms[i] = gdk_atom_intern (targets[i].target, FALSE);
+    gdk_win32_selection_add_targets (gtk_widget_get_window (widget), selection, ntargets, atoms);
+    g_free (atoms);
+  }
 #endif
 }
 
@@ -1140,15 +1092,6 @@ gtk_selection_convert (GtkWidget *widget,
   display = gtk_widget_get_display (widget);
   owner_window = gdk_selection_owner_get_for_display (display, selection);
   
-#ifdef GDK_WINDOWING_WIN32
-  /* Special handling for DELETE requests,
-   * make sure this goes down into GDK layer.
-   */
-  if (GDK_IS_WIN32_DISPLAY (display) &&
-      target == gdk_atom_intern_static_string ("DELETE"))
-    owner_window = NULL;
-#endif
-
   if (owner_window != NULL)
     {
       GtkWidget *owner_widget;
@@ -1184,25 +1127,6 @@ gtk_selection_convert (GtkWidget *widget,
 	  return TRUE;
 	}
     }
-
-#if defined GDK_WINDOWING_BROADWAY
-  /* This patch is a workaround to circumvent unimplemented
-     clipboard functionality in broadwayd. It eliminates
-     35s delay on popup menu before first clipboard copy,
-     by preventing conversion to be started.
-   
-     https://gitlab.gnome.org/GNOME/gtk/issues/1630
-  */ 
-  if (GDK_IS_BROADWAY_DISPLAY (display))
-  {
-      g_debug("gtk_selection_convert: disabled for broadway backend");
-
-      gtk_selection_retrieval_report (
-          info, GDK_NONE, 0, NULL, -1, GDK_CURRENT_TIME);
-
-      return FALSE;
-  }
-#endif
   
   /* Otherwise, we need to go through X */
   
@@ -1289,12 +1213,11 @@ gtk_selection_data_get_format (const GtkSelectionData *selection_data)
 
 /**
  * gtk_selection_data_get_data: (skip)
- * @selection_data: a pointer to a
- *   #GtkSelectionData-struct.
+ * @selection_data: a pointer to a #GtkSelectionData-struct.
  *
  * Retrieves the raw data of the selection.
  *
- * Returns: (array) (element-type guint8): the raw data of the selection.
+ * Returns: the raw data of the selection.
  *
  * Since: 2.14
  **/
@@ -1436,15 +1359,14 @@ selection_set_compound_text (GtkSelectionData *selection_data,
 			     const gchar      *str,
 			     gint              len)
 {
-  gboolean result = FALSE;
-
-#ifdef GDK_WINDOWING_X11
   gchar *tmp;
   guchar *text;
   GdkAtom encoding;
   gint format;
   gint new_length;
+  gboolean result = FALSE;
 
+#ifdef GDK_WINDOWING_X11
   if (GDK_IS_X11_DISPLAY (selection_data->display))
     {
       tmp = g_strndup (str, len);
@@ -1672,7 +1594,7 @@ gtk_selection_data_set_text (GtkSelectionData     *selection_data,
  * 
  * Gets the contents of the selection data as a UTF-8 string.
  * 
- * Returns: (type utf8) (nullable) (transfer full): if the selection data contained a
+ * Returns: (type utf8): if the selection data contained a
  *   recognized text type and it could be converted to UTF-8, a newly
  *   allocated string containing the converted text, otherwise %NULL.
  *   If the result is non-%NULL it must be freed with g_free().
@@ -1790,11 +1712,10 @@ gtk_selection_data_set_pixbuf (GtkSelectionData *selection_data,
  * 
  * Gets the contents of the selection data as a #GdkPixbuf.
  * 
- * Returns: (nullable) (transfer full): if the selection data
- *   contained a recognized image type and it could be converted to a
- *   #GdkPixbuf, a newly allocated pixbuf is returned, otherwise
- *   %NULL.  If the result is non-%NULL it must be freed with
- *   g_object_unref().
+ * Returns: (transfer full): if the selection data contained a recognized
+ *   image type and it could be converted to a #GdkPixbuf, a 
+ *   newly allocated pixbuf is returned, otherwise %NULL.
+ *   If the result is non-%NULL it must be freed with g_object_unref().
  *
  * Since: 2.6
  **/
@@ -2522,27 +2443,23 @@ _gtk_selection_request (GtkWidget *widget,
 	  info->conversions[i].data = data;
 	  info->num_incrs++;
 	  
-	  gdk_error_trap_push ();
 	  gdk_property_change (info->requestor, 
 			       info->conversions[i].property,
 			       gtk_selection_atoms[INCR],
 			       32,
 			       GDK_PROP_MODE_REPLACE,
 			       (guchar *)&items, 1);
-	  gdk_error_trap_pop_ignored ();
 	}
       else
 	{
 	  info->conversions[i].offset = -1;
 	  
-	  gdk_error_trap_push ();
 	  gdk_property_change (info->requestor, 
 			       info->conversions[i].property,
 			       data.type,
 			       data.format,
 			       GDK_PROP_MODE_REPLACE,
 			       data.data, items);
-	  gdk_error_trap_pop_ignored ();
 	  
 	  g_free (data.data);
 	}
@@ -2562,11 +2479,9 @@ _gtk_selection_request (GtkWidget *widget,
       g_message ("Starting INCR...");
 #endif
       
-      gdk_error_trap_push ();
       gdk_window_set_events (info->requestor,
 			     gdk_window_get_events (info->requestor) |
 			     GDK_PROPERTY_CHANGE_MASK);
-      gdk_error_trap_pop_ignored ();
       current_incrs = g_list_append (current_incrs, info);
       id = gdk_threads_add_timeout (1000, (GSourceFunc) gtk_selection_incr_timeout, info);
       g_source_set_name_by_id (id, "[gtk+] gtk_selection_incr_timeout");
@@ -2582,13 +2497,11 @@ _gtk_selection_request (GtkWidget *widget,
 	  mult_atoms[2*i] = info->conversions[i].target;
 	  mult_atoms[2*i+1] = info->conversions[i].property;
 	}
-
-      gdk_error_trap_push ();
+      
       gdk_property_change (info->requestor, event->property,
 			   gdk_atom_intern_static_string ("ATOM_PAIR"), 32, 
 			   GDK_PROP_MODE_REPLACE,
 			   (guchar *)mult_atoms, 2*info->num_conversions);
-      gdk_error_trap_pop_ignored ();
       g_free (mult_atoms);
     }
 
@@ -2712,14 +2625,12 @@ _gtk_selection_incr_event (GdkWindow	   *window,
 #endif
 
 	  bytes_per_item = gtk_selection_bytes_per_item (info->conversions[i].data.format);
-	  gdk_error_trap_push ();
 	  gdk_property_change (info->requestor, event->atom,
 			       info->conversions[i].data.type,
 			       info->conversions[i].data.format,
 			       GDK_PROP_MODE_REPLACE,
 			       buffer,
 			       num_bytes / bytes_per_item);
-	  gdk_error_trap_pop_ignored ();
 	  
 	  if (info->conversions[i].offset == -2)
 	    {
