@@ -14,9 +14,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA  02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  *
  * Author:
  *      Emmanuele Bassi <ebassi@linux.intel.com>
@@ -27,7 +25,7 @@
 
 /**
  * SECTION:gtkswitch
- * @Short_Description: A "light switch" style toggle
+ * @Short_Description: A “light switch” style toggle
  * @Title: GtkSwitch
  * @See_Also: #GtkToggleButton
  *
@@ -40,23 +38,26 @@
 
 #include "gtkswitch.h"
 
-#include "gtkaccessibleprivate.h"
-#include "gtkactivatable.h"
+#include "deprecated/gtkactivatable.h"
 #include "gtkintl.h"
 #include "gtkprivate.h"
-#include "gtktoggleaction.h"
+#include "deprecated/gtktoggleaction.h"
 #include "gtkwidget.h"
 #include "gtkmarshalers.h"
+#include "gtkapplicationprivate.h"
+#include "gtkactionable.h"
+#include "a11y/gtkswitchaccessible.h"
+#include "gtkactionhelper.h"
 
 #include <math.h>
 
 #define DEFAULT_SLIDER_WIDTH    (36)
-#define DEFAULT_SLIDER_HEIGHT   (22)
 
 struct _GtkSwitchPrivate
 {
   GdkWindow *event_window;
   GtkAction *action;
+  GtkActionHelper *action_helper;
 
   gint handle_x;
   gint offset;
@@ -76,7 +77,9 @@ enum
   PROP_ACTIVE,
   PROP_RELATED_ACTION,
   PROP_USE_ACTION_APPEARANCE,
-  LAST_PROP
+  LAST_PROP,
+  PROP_ACTION_NAME,
+  PROP_ACTION_TARGET
 };
 
 enum
@@ -89,13 +92,17 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 static GParamSpec *switch_props[LAST_PROP] = { NULL, };
 
-static GType gtk_switch_accessible_factory_get_type (void);
-
+static void gtk_switch_actionable_iface_init (GtkActionableInterface *iface);
 static void gtk_switch_activatable_interface_init (GtkActivatableIface *iface);
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
 G_DEFINE_TYPE_WITH_CODE (GtkSwitch, gtk_switch, GTK_TYPE_WIDGET,
+                         G_ADD_PRIVATE (GtkSwitch)
+                         G_IMPLEMENT_INTERFACE (GTK_TYPE_ACTIONABLE,
+                                                gtk_switch_actionable_iface_init)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_ACTIVATABLE,
                                                 gtk_switch_activatable_interface_init));
+G_GNUC_END_IGNORE_DEPRECATIONS;
 
 static gboolean
 gtk_switch_button_press (GtkWidget      *widget,
@@ -104,6 +111,10 @@ gtk_switch_button_press (GtkWidget      *widget,
   GtkSwitchPrivate *priv = GTK_SWITCH (widget)->priv;
   GtkAllocation allocation;
 
+  /* Don't handle extra mouse buttons events, let them bubble up */
+  if (event->button > 5)
+    return GDK_EVENT_PROPAGATE;
+
   gtk_widget_get_allocation (widget, &allocation);
 
   if (priv->is_active)
@@ -114,7 +125,7 @@ gtk_switch_button_press (GtkWidget      *widget,
       if (event->x <= allocation.width / 2)
         {
           priv->in_press = TRUE;
-          return FALSE;
+          return GDK_EVENT_STOP;
         }
 
       priv->offset = event->x - allocation.width / 2;
@@ -127,7 +138,7 @@ gtk_switch_button_press (GtkWidget      *widget,
       if (event->x >= allocation.width / 2)
         {
           priv->in_press = TRUE;
-          return FALSE;
+          return GDK_EVENT_STOP;
         }
 
       priv->offset = event->x;
@@ -139,7 +150,7 @@ gtk_switch_button_press (GtkWidget      *widget,
                 "gtk-dnd-drag-threshold", &priv->drag_threshold,
                 NULL);
 
-  return FALSE;
+  return GDK_EVENT_STOP;
 }
 
 static gboolean
@@ -150,10 +161,10 @@ gtk_switch_motion (GtkWidget      *widget,
 
   /* if this is a direct toggle we don't handle motion */
   if (priv->in_press)
-    return FALSE;
+    return GDK_EVENT_PROPAGATE;
 
   if (ABS (event->x - priv->drag_start) < priv->drag_threshold)
-    return TRUE;
+    return GDK_EVENT_STOP;
 
   if (event->state & GDK_BUTTON1_MASK)
     {
@@ -194,10 +205,10 @@ gtk_switch_motion (GtkWidget      *widget,
       /* we need to redraw the handle */
       gtk_widget_queue_draw (widget);
 
-      return TRUE;
+      return GDK_EVENT_STOP;
     }
 
-  return FALSE;
+  return GDK_EVENT_PROPAGATE;
 }
 
 static gboolean
@@ -207,6 +218,10 @@ gtk_switch_button_release (GtkWidget      *widget,
   GtkSwitchPrivate *priv = GTK_SWITCH (widget)->priv;
   GtkAllocation allocation;
 
+  /* Don't handle extra mouse buttons events, let them bubble up */
+  if (event->button > 5)
+    return GDK_EVENT_PROPAGATE;
+
   gtk_widget_get_allocation (widget, &allocation);
 
   /* dragged toggles have a "soft" grab, so we don't care whether we
@@ -214,7 +229,7 @@ gtk_switch_button_release (GtkWidget      *widget,
    * for direct toggles, instead
    */
   if (!priv->is_dragging && !priv->in_switch)
-    return FALSE;
+    return GDK_EVENT_PROPAGATE;
 
   /* direct toggle */
   if (priv->in_press)
@@ -222,7 +237,7 @@ gtk_switch_button_release (GtkWidget      *widget,
       priv->in_press = FALSE;
       gtk_switch_set_active (GTK_SWITCH (widget), !priv->is_active);
 
-      return TRUE;
+      return GDK_EVENT_STOP;
     }
 
   /* toggle the switch if the handle was clicked but a drag had not been
@@ -231,7 +246,7 @@ gtk_switch_button_release (GtkWidget      *widget,
     {
       gtk_switch_set_active (GTK_SWITCH (widget), !priv->is_active);
 
-      return TRUE;
+      return GDK_EVENT_STOP;
     }
 
   /* dragged toggle */
@@ -255,10 +270,10 @@ gtk_switch_button_release (GtkWidget      *widget,
 
       gtk_widget_queue_draw (widget);
 
-      return TRUE;
+      return GDK_EVENT_STOP;
     }
 
-  return FALSE;
+  return GDK_EVENT_PROPAGATE;
 }
 
 static gboolean
@@ -298,7 +313,6 @@ gtk_switch_get_preferred_width (GtkWidget *widget,
                                 gint      *minimum,
                                 gint      *natural)
 {
-  GtkSwitchPrivate *priv = GTK_SWITCH (widget)->priv;
   GtkStyleContext *context;
   GtkStateFlags state;
   GtkBorder padding;
@@ -307,14 +321,10 @@ gtk_switch_get_preferred_width (GtkWidget *widget,
   PangoRectangle logical_rect;
 
   context = gtk_widget_get_style_context (widget);
-  state = gtk_widget_get_state_flags (widget);
-
-  if (priv->is_active)
-    state |= GTK_STATE_FLAG_ACTIVE;
+  state = gtk_style_context_get_state (context);
 
   gtk_style_context_save (context);
 
-  gtk_style_context_set_state (context, state);
   gtk_style_context_add_class (context, GTK_STYLE_CLASS_SLIDER);
   gtk_style_context_get_padding (context, state, &padding);
 
@@ -328,7 +338,7 @@ gtk_switch_get_preferred_width (GtkWidget *widget,
                         "focus-padding", &focus_pad,
                         NULL);
 
-  width += 2 * (focus_width + focus_pad);
+  slider_width = MAX (slider_width, 3 * (focus_width + focus_pad));
 
   /* Translators: if the "on" state label requires more than three
    * glyphs then use MEDIUM VERTICAL BAR (U+2759) as the text for
@@ -361,24 +371,19 @@ gtk_switch_get_preferred_height (GtkWidget *widget,
                                  gint      *minimum,
                                  gint      *natural)
 {
-  GtkSwitchPrivate *priv = GTK_SWITCH (widget)->priv;
   GtkStyleContext *context;
   GtkStateFlags state;
   GtkBorder padding;
-  gint height, focus_width, focus_pad;
+  gint height, focus_width, focus_pad, slider_width, min_height;
   PangoLayout *layout;
   PangoRectangle logical_rect;
   gchar *str;
 
   context = gtk_widget_get_style_context (widget);
-  state = gtk_widget_get_state_flags (widget);
-
-  if (priv->is_active)
-    state |= GTK_STATE_FLAG_ACTIVE;
+  state = gtk_style_context_get_state (context);
 
   gtk_style_context_save (context);
 
-  gtk_style_context_set_state (context, state);
   gtk_style_context_add_class (context, GTK_STYLE_CLASS_SLIDER);
   gtk_style_context_get_padding (context, state, &padding);
 
@@ -387,11 +392,12 @@ gtk_switch_get_preferred_height (GtkWidget *widget,
   gtk_style_context_restore (context);
 
   gtk_widget_style_get (widget,
+                        "slider-width", &slider_width,
                         "focus-line-width", &focus_width,
                         "focus-padding", &focus_pad,
                         NULL);
 
-  height += 2 * (focus_width + focus_pad);
+  min_height = MAX (slider_width * 0.6, 3 * (focus_width + focus_pad));
 
   str = g_strdup_printf ("%s%s",
                          C_("switch", "ON"),
@@ -400,7 +406,7 @@ gtk_switch_get_preferred_height (GtkWidget *widget,
   layout = gtk_widget_create_pango_layout (widget, str);
   pango_layout_get_extents (layout, NULL, &logical_rect);
   pango_extents_to_pixels (&logical_rect, NULL);
-  height += MAX (DEFAULT_SLIDER_HEIGHT, logical_rect.height);
+  height += MAX (min_height, logical_rect.height);
 
   g_object_unref (layout);
   g_free (str);
@@ -463,7 +469,7 @@ gtk_switch_realize (GtkWidget *widget)
   priv->event_window = gdk_window_new (parent_window,
                                        &attributes,
                                        attributes_mask);
-  gdk_window_set_user_data (priv->event_window, widget);
+  gtk_widget_register_window (widget, priv->event_window);
 }
 
 static void
@@ -473,7 +479,7 @@ gtk_switch_unrealize (GtkWidget *widget)
 
   if (priv->event_window != NULL)
     {
-      gdk_window_set_user_data (priv->event_window, NULL);
+      gtk_widget_unregister_window (widget, priv->event_window);
       gdk_window_destroy (priv->event_window);
       priv->event_window = NULL;
     }
@@ -508,17 +514,9 @@ gtk_switch_paint_handle (GtkWidget    *widget,
                          cairo_t      *cr,
                          GdkRectangle *box)
 {
-  GtkSwitchPrivate *priv = GTK_SWITCH (widget)->priv;
   GtkStyleContext *context = gtk_widget_get_style_context (widget);
-  GtkStateFlags state;
-
-  state = gtk_widget_get_state_flags (widget);
-
-  if (priv->is_active)
-    state |= GTK_STATE_FLAG_ACTIVE;
 
   gtk_style_context_save (context);
-  gtk_style_context_set_state (context, state);
   gtk_style_context_add_class (context, GTK_STYLE_CLASS_SLIDER);
 
   gtk_render_slider (context, cr,
@@ -537,30 +535,17 @@ gtk_switch_draw (GtkWidget *widget,
   GtkStyleContext *context;
   GdkRectangle handle;
   PangoLayout *layout;
-  PangoFontDescription *desc;
-  const PangoFontDescription *style_desc;
   PangoRectangle rect;
   gint label_x, label_y;
-  GtkStateFlags state;
   GtkBorder padding;
-  gint focus_width, focus_pad;
+  GtkStateFlags state;
   gint x, y, width, height;
-  gint font_size, style_font_size;
-
-  gtk_widget_style_get (widget,
-                        "focus-line-width", &focus_width,
-                        "focus-padding", &focus_pad,
-                        NULL);
 
   context = gtk_widget_get_style_context (widget);
   state = gtk_widget_get_state_flags (widget);
 
-  if (priv->is_active)
-    state |= GTK_STATE_FLAG_ACTIVE;
-
   gtk_style_context_save (context);
 
-  gtk_style_context_set_state (context, state);
   gtk_style_context_add_class (context, GTK_STYLE_CLASS_SLIDER);
 
   gtk_style_context_get_padding (context, state, &padding);
@@ -572,17 +557,8 @@ gtk_switch_draw (GtkWidget *widget,
   width = gtk_widget_get_allocated_width (widget);
   height = gtk_widget_get_allocated_height (widget);
 
-  if (gtk_widget_has_focus (widget))
-    gtk_render_focus (context, cr, x, y, width, height);
-
-  x += focus_width + focus_pad;
-  y += focus_width + focus_pad;
-  width -= 2 * (focus_width + focus_pad);
-  height -= 2 * (focus_width + focus_pad);
-
   gtk_style_context_save (context);
   gtk_style_context_add_class (context, GTK_STYLE_CLASS_TROUGH);
-  gtk_style_context_set_state (context, state);
 
   gtk_render_background (context, cr, x, y, width, height);
   gtk_render_frame (context, cr, x, y, width, height);
@@ -603,20 +579,6 @@ gtk_switch_draw (GtkWidget *widget,
    */
   layout = gtk_widget_create_pango_layout (widget, C_("switch", "ON"));
 
-  /* FIXME: this should be really done in the theme, but overriding font size
-   * from it doesn't currently work. So we have to hardcode this here and
-   * below for the "OFF" label.
-   */
-  desc = pango_font_description_new ();
-
-  style_desc = gtk_style_context_get_font (context, state);
-  style_font_size = pango_font_description_get_size (style_desc);
-  font_size = MAX (style_font_size - 1 * PANGO_SCALE, ceil (style_font_size * PANGO_SCALE_SMALL));
-
-  pango_font_description_set_size (desc, font_size);
-
-  pango_layout_set_font_description (layout, desc);
-
   pango_layout_get_extents (layout, NULL, &rect);
   pango_extents_to_pixels (&rect, NULL);
 
@@ -631,7 +593,6 @@ gtk_switch_draw (GtkWidget *widget,
    * glyphs then use WHITE CIRCLE (U+25CB) as the text for the state
    */
   layout = gtk_widget_create_pango_layout (widget, C_("switch", "OFF"));
-  pango_layout_set_font_description (layout, desc);
 
   pango_layout_get_extents (layout, NULL, &rect);
   pango_extents_to_pixels (&rect, NULL);
@@ -654,24 +615,23 @@ gtk_switch_draw (GtkWidget *widget,
 
   gtk_switch_paint_handle (widget, cr, &handle);
 
-  pango_font_description_free (desc);
-
-  return FALSE;
-}
-
-static AtkObject *
-gtk_switch_get_accessible (GtkWidget *widget)
-{
-  static gboolean first_time = TRUE;
-
-  if (G_UNLIKELY (first_time))
+  if (gtk_widget_has_visible_focus (widget))
     {
-      _gtk_accessible_set_factory_type (GTK_TYPE_SWITCH,
-                                        gtk_switch_accessible_factory_get_type ());
-      first_time = FALSE;
+      gint focus_width, focus_pad, pad;
+
+      gtk_widget_style_get (widget,
+                            "focus-line-width", &focus_width,
+                            "focus-padding", &focus_pad,
+                            NULL);
+
+      pad = focus_pad + focus_width;
+
+      gtk_render_focus (context, cr,
+                        handle.x + pad, handle.y + pad,
+                        handle.width - pad*2, handle.height - pad*2);
     }
 
-  return GTK_WIDGET_CLASS (gtk_switch_parent_class)->get_accessible (widget);
+  return FALSE;
 }
 
 static void
@@ -683,7 +643,9 @@ gtk_switch_set_related_action (GtkSwitch *sw,
   if (priv->action == action)
     return;
 
+  G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
   gtk_activatable_do_set_related_action (GTK_ACTIVATABLE (sw), action);
+  G_GNUC_END_IGNORE_DEPRECATIONS;
 
   priv->action = action;
 }
@@ -698,10 +660,61 @@ gtk_switch_set_use_action_appearance (GtkSwitch *sw,
     {
       priv->use_action_appearance = use_appearance;
 
+      G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
       gtk_activatable_sync_action_properties (GTK_ACTIVATABLE (sw), priv->action);
+      G_GNUC_END_IGNORE_DEPRECATIONS;
     }
 }
 
+static void
+gtk_switch_set_action_name (GtkActionable *actionable,
+                            const gchar   *action_name)
+{
+  GtkSwitch *sw = GTK_SWITCH (actionable);
+
+  if (!sw->priv->action_helper)
+    sw->priv->action_helper = gtk_action_helper_new (actionable);
+
+  gtk_action_helper_set_action_name (sw->priv->action_helper, action_name);
+}
+
+static void
+gtk_switch_set_action_target_value (GtkActionable *actionable,
+                                    GVariant      *action_target)
+{
+  GtkSwitch *sw = GTK_SWITCH (actionable);
+
+  if (!sw->priv->action_helper)
+    sw->priv->action_helper = gtk_action_helper_new (actionable);
+
+  gtk_action_helper_set_action_target_value (sw->priv->action_helper, action_target);
+}
+
+static const gchar *
+gtk_switch_get_action_name (GtkActionable *actionable)
+{
+  GtkSwitch *sw = GTK_SWITCH (actionable);
+
+  return gtk_action_helper_get_action_name (sw->priv->action_helper);
+}
+
+static GVariant *
+gtk_switch_get_action_target_value (GtkActionable *actionable)
+{
+  GtkSwitch *sw = GTK_SWITCH (actionable);
+
+  return gtk_action_helper_get_action_target_value (sw->priv->action_helper);
+}
+
+static void
+gtk_switch_actionable_iface_init (GtkActionableInterface *iface)
+{
+  iface->get_action_name = gtk_switch_get_action_name;
+  iface->set_action_name = gtk_switch_set_action_name;
+  iface->get_action_target_value = gtk_switch_get_action_target_value;
+  iface->set_action_target_value = gtk_switch_set_action_target_value;
+}
+
 static void
 gtk_switch_set_property (GObject      *gobject,
                          guint         prop_id,
@@ -724,6 +737,14 @@ gtk_switch_set_property (GObject      *gobject,
       gtk_switch_set_use_action_appearance (sw, g_value_get_boolean (value));
       break;
 
+    case PROP_ACTION_NAME:
+      gtk_switch_set_action_name (GTK_ACTIONABLE (sw), g_value_get_string (value));
+      break;
+
+    case PROP_ACTION_TARGET:
+      gtk_switch_set_action_target_value (GTK_ACTIONABLE (sw), g_value_get_variant (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
     }
@@ -751,6 +772,14 @@ gtk_switch_get_property (GObject    *gobject,
       g_value_set_boolean (value, priv->use_action_appearance);
       break;
 
+    case PROP_ACTION_NAME:
+      g_value_set_string (value, gtk_action_helper_get_action_name (priv->action_helper));
+      break;
+
+    case PROP_ACTION_TARGET:
+      g_value_set_variant (value, gtk_action_helper_get_action_target_value (priv->action_helper));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
     }
@@ -761,9 +790,13 @@ gtk_switch_dispose (GObject *object)
 {
   GtkSwitchPrivate *priv = GTK_SWITCH (object)->priv;
 
+  g_clear_object (&priv->action_helper);
+
   if (priv->action)
     {
+      G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
       gtk_activatable_do_set_related_action (GTK_ACTIVATABLE (object), NULL);
+      G_GNUC_END_IGNORE_DEPRECATIONS;
       priv->action = NULL;
     }
 
@@ -777,9 +810,10 @@ gtk_switch_class_init (GtkSwitchClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   gpointer activatable_iface;
 
-  g_type_class_add_private (klass, sizeof (GtkSwitchPrivate));
-
+  G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
   activatable_iface = g_type_default_interface_peek (GTK_TYPE_ACTIVATABLE);
+  G_GNUC_END_IGNORE_DEPRECATIONS;
+
   switch_props[PROP_RELATED_ACTION] =
     g_param_spec_override ("related-action",
                            g_object_interface_find_property (activatable_iface,
@@ -821,7 +855,6 @@ gtk_switch_class_init (GtkSwitchClass *klass)
   widget_class->motion_notify_event = gtk_switch_motion;
   widget_class->enter_notify_event = gtk_switch_enter;
   widget_class->leave_notify_event = gtk_switch_leave;
-  widget_class->get_accessible = gtk_switch_get_accessible;
 
   klass->activate = gtk_switch_activate;
 
@@ -857,12 +890,17 @@ gtk_switch_class_init (GtkSwitchClass *klass)
                   G_TYPE_NONE, 0);
   widget_class->activate_signal = signals[ACTIVATE];
 
+  g_object_class_override_property (gobject_class, PROP_ACTION_NAME, "action-name");
+  g_object_class_override_property (gobject_class, PROP_ACTION_TARGET, "action-target");
+
+  gtk_widget_class_set_accessible_type (widget_class, GTK_TYPE_SWITCH_ACCESSIBLE);
+  gtk_widget_class_set_accessible_role (widget_class, ATK_ROLE_TOGGLE_BUTTON);
 }
 
 static void
 gtk_switch_init (GtkSwitch *self)
 {
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTK_TYPE_SWITCH, GtkSwitchPrivate);
+  self->priv = gtk_switch_get_instance_private (self);
   self->priv->use_action_appearance = TRUE;
   gtk_widget_set_has_window (GTK_WIDGET (self), FALSE);
   gtk_widget_set_can_focus (GTK_WIDGET (self), TRUE);
@@ -873,7 +911,7 @@ gtk_switch_init (GtkSwitch *self)
  *
  * Creates a new #GtkSwitch widget.
  *
- * Return value: the newly created #GtkSwitch instance
+ * Returns: the newly created #GtkSwitch instance
  *
  * Since: 3.0
  */
@@ -907,27 +945,26 @@ gtk_switch_set_active (GtkSwitch *sw,
   if (priv->is_active != is_active)
     {
       AtkObject *accessible;
-      GtkWidget *widget;
-      GtkStyleContext *context;
 
-      widget = GTK_WIDGET (sw);
       priv->is_active = is_active;
 
       g_object_notify_by_pspec (G_OBJECT (sw), switch_props[PROP_ACTIVE]);
 
+      if (priv->action_helper)
+        gtk_action_helper_activate (priv->action_helper);
+
+      G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
       if (priv->action)
         gtk_action_activate (priv->action);
+      G_GNUC_END_IGNORE_DEPRECATIONS;
 
       accessible = gtk_widget_get_accessible (GTK_WIDGET (sw));
       atk_object_notify_state_change (accessible, ATK_STATE_CHECKED, priv->is_active);
 
-      if (gtk_widget_get_realized (widget))
-        {
-          context = gtk_widget_get_style_context (widget);
-          gtk_style_context_notify_state_change (context,
-                                                 gtk_widget_get_window (widget),
-                                                 NULL, GTK_STATE_ACTIVE, is_active);
-        }
+      if (priv->is_active)
+        gtk_widget_set_state_flags (GTK_WIDGET (sw), GTK_STATE_FLAG_ACTIVE, FALSE);
+      else
+        gtk_widget_unset_state_flags (GTK_WIDGET (sw), GTK_STATE_FLAG_ACTIVE);
 
       gtk_widget_queue_draw (GTK_WIDGET (sw));
     }
@@ -937,9 +974,9 @@ gtk_switch_set_active (GtkSwitch *sw,
  * gtk_switch_get_active:
  * @sw: a #GtkSwitch
  *
- * Gets whether the #GtkSwitch is in its "on" or "off" state.
+ * Gets whether the #GtkSwitch is in its “on” or “off” state.
  *
- * Return value: %TRUE if the #GtkSwitch is active, and %FALSE otherwise
+ * Returns: %TRUE if the #GtkSwitch is active, and %FALSE otherwise
  *
  * Since: 3.0
  */
@@ -956,6 +993,8 @@ gtk_switch_update (GtkActivatable *activatable,
                    GtkAction      *action,
                    const gchar    *property_name)
 {
+  G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
+
   if (strcmp (property_name, "visible") == 0)
     {
       if (gtk_action_is_visible (action))
@@ -973,6 +1012,8 @@ gtk_switch_update (GtkActivatable *activatable,
       gtk_switch_set_active (GTK_SWITCH (activatable), gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
       gtk_action_unblock_activate (action);
     }
+
+  G_GNUC_END_IGNORE_DEPRECATIONS;
 }
 
 static void
@@ -982,6 +1023,8 @@ gtk_switch_sync_action_properties (GtkActivatable *activatable,
   if (!action)
     return;
 
+  G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
+
   if (gtk_action_is_visible (action))
     gtk_widget_show (GTK_WIDGET (activatable));
   else
@@ -992,6 +1035,8 @@ gtk_switch_sync_action_properties (GtkActivatable *activatable,
   gtk_action_block_activate (action);
   gtk_switch_set_active (GTK_SWITCH (activatable), gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
   gtk_action_unblock_activate (action);
+
+  G_GNUC_END_IGNORE_DEPRECATIONS;
 }
 
 static void
@@ -1000,210 +1045,3 @@ gtk_switch_activatable_interface_init (GtkActivatableIface *iface)
   iface->update = gtk_switch_update;
   iface->sync_action_properties = gtk_switch_sync_action_properties;
 }
-
-/* accessibility: object */
-
-typedef struct _GtkSwitchAccessible      GtkSwitchAccessible;
-typedef struct _GtkSwitchAccessibleClass GtkSwitchAccessibleClass;
-
-struct _GtkSwitchAccessible
-{
-  GtkAccessible object;
-
-  gchar *description;
-  guint  action_idle;
-};
-
-static void atk_action_interface_init (AtkActionIface *iface);
-
-ATK_DEFINE_TYPE_WITH_CODE (GtkSwitchAccessible, _gtk_switch_accessible, GTK_TYPE_SWITCH,
-                           G_IMPLEMENT_INTERFACE (ATK_TYPE_ACTION, atk_action_interface_init))
-
-static AtkStateSet *
-gtk_switch_accessible_ref_state_set (AtkObject *accessible)
-{
-  AtkStateSet *state_set;
-  GtkWidget *widget;
-
-  state_set = ATK_OBJECT_CLASS (_gtk_switch_accessible_parent_class)->ref_state_set (accessible);
-
-  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (accessible));
-  if (widget == NULL)
-    return state_set;
-
-  if (gtk_switch_get_active (GTK_SWITCH (widget)))
-    atk_state_set_add_state (state_set, ATK_STATE_CHECKED);
-
-  return state_set;
-}
-
-static void
-gtk_switch_accessible_finalize (GObject *obj)
-{
-  GtkSwitchAccessible *accessible = (GtkSwitchAccessible *)obj;
-
-  g_free (accessible->description);
-
-  if (accessible->action_idle)
-    g_source_remove (accessible->action_idle);
-
-  G_OBJECT_CLASS (_gtk_switch_accessible_parent_class)->finalize (obj);
-}
-
-static void
-_gtk_switch_accessible_initialize (AtkObject *accessible,
-                                   gpointer   widget)
-{
-  ATK_OBJECT_CLASS (_gtk_switch_accessible_parent_class)->initialize (accessible, widget);
-
-  atk_object_set_role (accessible, ATK_ROLE_TOGGLE_BUTTON);
-  atk_object_set_name (accessible, C_("light switch widget", "Switch"));
-  atk_object_set_description (accessible, _("Switches between on and off states"));
-}
-
-static void
-_gtk_switch_accessible_class_init (GtkSwitchAccessibleClass *klass)
-{
-  GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  AtkObjectClass *atk_class = ATK_OBJECT_CLASS (klass);
-
-  object_class->finalize = gtk_switch_accessible_finalize;
-
-  atk_class->initialize = _gtk_switch_accessible_initialize;
-  atk_class->ref_state_set = gtk_switch_accessible_ref_state_set;
-}
-
-static void
-_gtk_switch_accessible_init (GtkSwitchAccessible *self)
-{
-  self->description = NULL;
-  self->action_idle = 0;
-}
-
-/* accessibility: action interface */
-
-static gint
-gtk_switch_action_get_n_actions (AtkAction *action)
-{
-  return 1;
-}
-
-static const gchar *
-gtk_switch_action_get_name (AtkAction *action,
-                            gint       i)
-{
-  return "toggle";
-}
-
-static const gchar *
-gtk_switch_action_get_description (AtkAction *action,
-                                   gint       i)
-{
-  GtkSwitchAccessible *accessible = (GtkSwitchAccessible*)action;
-
-  return accessible->description;
-}
-
-static gboolean
-gtk_switch_action_set_description (AtkAction   *action,
-                                   gint         i,
-                                   const gchar *description)
-{
-  GtkSwitchAccessible *accessible = (GtkSwitchAccessible*)action;
-
-  g_free (accessible->description);
-  accessible->description = g_strdup (description);
-
-  return TRUE;
-}
-
-static gboolean
-idle_do_action (gpointer data)
-{
-  GtkSwitchAccessible *accessible = data;
-  GtkWidget *widget;
-  GtkSwitch *sw;
-
-  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (data));
-  sw = GTK_SWITCH (widget);
-
-  accessible->action_idle = 0;
-
-  if (widget == NULL ||
-      !gtk_widget_is_sensitive (widget) || !gtk_widget_get_visible (widget))
-    return FALSE;
-
-  gtk_switch_set_active (sw, !gtk_switch_get_active (sw));
-
-  return FALSE;
-}
-
-static gboolean
-gtk_switch_action_do_action (AtkAction *action,
-                             gint       i)
-{
-  GtkSwitchAccessible *accessible;
-  GtkWidget *widget;
-
-  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (action));
-  if (widget == NULL)
-    return FALSE;
-
-  if (!gtk_widget_is_sensitive (widget) || !gtk_widget_get_visible (widget))
-    return FALSE;
-
-  accessible = (GtkSwitchAccessible *)action;
-
-  if (!accessible->action_idle)
-    accessible->action_idle = gdk_threads_add_idle (idle_do_action, accessible);
-
-  return TRUE;
-}
-
-static void
-atk_action_interface_init (AtkActionIface *iface)
-{
-  iface->do_action = gtk_switch_action_do_action;
-  iface->get_n_actions = gtk_switch_action_get_n_actions;
-  iface->get_name = gtk_switch_action_get_name;
-  iface->get_description = gtk_switch_action_get_description;
-  iface->set_description = gtk_switch_action_set_description;
-}
-
-/* accessibility: factory */
-
-typedef AtkObjectFactoryClass   GtkSwitchAccessibleFactoryClass;
-typedef AtkObjectFactory        GtkSwitchAccessibleFactory;
-
-G_DEFINE_TYPE (GtkSwitchAccessibleFactory,
-               gtk_switch_accessible_factory,
-               ATK_TYPE_OBJECT_FACTORY);
-
-static GType
-gtk_switch_accessible_factory_get_accessible_type (void)
-{
-  return _gtk_switch_accessible_get_type ();
-}
-
-static AtkObject *
-gtk_switch_accessible_factory_create_accessible (GObject *obj)
-{
-  AtkObject *accessible;
-
-  accessible = g_object_new (_gtk_switch_accessible_get_type (), NULL);
-  atk_object_initialize (accessible, obj);
-
-  return accessible;
-}
-
-static void
-gtk_switch_accessible_factory_class_init (AtkObjectFactoryClass *klass)
-{
-  klass->create_accessible = gtk_switch_accessible_factory_create_accessible;
-  klass->get_accessible_type = gtk_switch_accessible_factory_get_accessible_type;
-}
-
-static void
-gtk_switch_accessible_factory_init (AtkObjectFactory *factory)
-{
-}
