@@ -38,6 +38,13 @@
 G_BEGIN_DECLS
 
 /* Kinds of widget-specific help */
+/**
+ * GtkWidgetHelpType:
+ * @GTK_WIDGET_HELP_TOOLTIP: Tooltip.
+ * @GTK_WIDGET_HELP_WHATS_THIS: What’s this.
+ *
+ * Kinds of widget-specific help. Used by the ::show-help signal.
+ */
 typedef enum
 {
   GTK_WIDGET_HELP_TOOLTIP,
@@ -45,7 +52,7 @@ typedef enum
 } GtkWidgetHelpType;
 
 /* Macro for casting a pointer to a GtkWidget or GtkWidgetClass pointer.
- * Macros for testing whether `widget' or `klass' are of type GTK_TYPE_WIDGET.
+ * Macros for testing whether widget or klass are of type GTK_TYPE_WIDGET.
  */
 #define GTK_TYPE_WIDGET			  (gtk_widget_get_type ())
 #define GTK_WIDGET(widget)		  (G_TYPE_CHECK_INSTANCE_CAST ((widget), GTK_TYPE_WIDGET, GtkWidget))
@@ -63,14 +70,15 @@ typedef struct _GtkWidgetAuxInfo       GtkWidgetAuxInfo;
 
 /**
  * GtkAllocation:
- * @x: the X position of the widget's area relative to its parents allocation.
- * @y: the Y position of the widget's area relative to its parents allocation.
- * @width: the width of the widget's allocated area.
- * @height: the height of the widget's allocated area.
+ * @x: the X position of the widget’s area relative to its parents allocation.
+ * @y: the Y position of the widget’s area relative to its parents allocation.
+ * @width: the width of the widget’s allocated area.
+ * @height: the height of the widget’s allocated area.
  *
- * A <structname>GtkAllocation</structname> of a widget represents region
+ * A #GtkAllocation-struct of a widget represents region
  * which has been allocated to the widget by its parent. It is a subregion
- * of its parents allocation. See <xref linkend="geometry-management"/> for
+ * of its parents allocation. See
+ * [GtkWidget’s geometry management section][geometry-management] for
  * more information.
  */
 typedef 	GdkRectangle	   GtkAllocation;
@@ -105,11 +113,12 @@ typedef gboolean (*GtkTickCallback) (GtkWidget     *widget,
 
 /**
  * GtkRequisition:
- * @width: the widget's desired width
- * @height: the widget's desired height
+ * @width: the widget’s desired width
+ * @height: the widget’s desired height
  *
- * A <structname>GtkRequisition</structname> represents the desired size of a widget. See
- * <xref linkend="geometry-management"/> for more information.
+ * A #GtkRequisition-struct represents the desired size of a widget. See
+ * [GtkWidget’s geometry management section][geometry-management] for
+ * more information.
  */
 struct _GtkRequisition
 {
@@ -126,6 +135,8 @@ struct _GtkWidget
 {
   GInitiallyUnowned parent_instance;
 
+  /*< private >*/
+
   GtkWidgetPrivate *priv;
 };
 
@@ -138,6 +149,46 @@ struct _GtkWidget
  * @activate_signal: The signal to emit when a widget of this class is
  *   activated, gtk_widget_activate() handles the emission.
  *   Implementation of this signal is optional.
+ * @dispatch_child_properties_changed: Seldomly overidden.
+ * @destroy: Signals that all holders of a reference to the widget
+ *   should release the reference that they hold.
+ * @show: Signal emitted when widget is shown
+ * @show_all: Recursively shows a widget, and any child widgets (if the widget is
+ * a container).
+ * @hide: Signal emitted when widget is hidden.
+ * @map: Signal emitted when widget is going to be mapped, that is
+ *   when the widget is visible (which is controlled with
+ *   gtk_widget_set_visible()) and all its parents up to the toplevel
+ *   widget are also visible.
+ * @unmap: Signal emitted when widget is going to be unmapped, which
+ *   means that either it or any of its parents up to the toplevel
+ *   widget have been set as hidden.
+ * @realize: Signal emitted when widget is associated with a
+ *   #GdkWindow, which means that gtk_widget_realize() has been called or
+ *   the widget has been mapped (that is, it is going to be drawn).
+ * @unrealize: Signal emitted when the GdkWindow associated with
+ *   widget is destroyed, which means that gtk_widget_unrealize() has
+ *   been called or the widget has been unmapped (that is, it is going
+ *   to be hidden).
+ * @size_allocate: Signal emitted to get the widget allocation.
+ * @state_changed: Signal emitted when the widget state
+ *   changes. Deprecated: 3.0
+ * @state_flags_changed: Signal emitted when the widget state changes,
+ *   see gtk_widget_get_state_flags().
+ * @parent_set: Signal emitted when a new parent has been set on a
+ *   widget.
+ * @hierarchy_changed: Signal emitted when the anchored state of a
+ *   widget changes.
+ * @style_set: Signal emitted when a new style has been set on a
+ * widget. Deprecated: 3.0
+ * @direction_changed: Signal emitted when the text direction of a
+ *   widget changes.
+ * @grab_notify: Signal emitted when a widget becomes shadowed by a
+ *   GTK+ grab (not a pointer or keyboard grab) on another widget, or
+ *   when it becomes unshadowed due to a grab being removed.
+ * @child_notify: Signal emitted for each child property that has
+ *   changed on an object.
+ * @draw: Signal emitted when a widget is supposed to render itself.
  * @get_request_mode: This allows a widget to tell its parent container whether
  *   it prefers to be allocated in %GTK_SIZE_REQUEST_HEIGHT_FOR_WIDTH or
  *   %GTK_SIZE_REQUEST_WIDTH_FOR_HEIGHT mode.
@@ -147,7 +198,7 @@ struct _GtkWidget
  *   %GTK_SIZE_REQUEST_CONSTANT_SIZE disables any height-for-width or
  *   width-for-height geometry management for a said widget and is the
  *   default return.
- *   It's important to note (as described below) that any widget
+ *   It’s important to note (as described below) that any widget
  *   which trades height-for-width or width-for-height must respond properly 
  *   to both of the virtual methods #GtkWidgetClass.get_preferred_height_for_width()
  *   and #GtkWidgetClass.get_preferred_width_for_height() since it might be 
@@ -159,14 +210,14 @@ struct _GtkWidget
  *   #GtkWidgetClass.get_preferred_height()).
  * @get_preferred_width_for_height: This is analogous to
  *   #GtkWidgetClass.get_preferred_height_for_width() except that it
- *   operates in the oposite orientation. It's rare that a widget actually
+ *   operates in the oposite orientation. It’s rare that a widget actually
  *   does %GTK_SIZE_REQUEST_WIDTH_FOR_HEIGHT requests but this can happen
  *   when, for example, a widget or container gets additional columns to
  *   compensate for a smaller allocated height.
  * @get_preferred_width: This is called by containers to obtain the minimum
  *   and natural width of a widget. A widget will never be allocated a width
  *   less than its minimum and will only ever be allocated a width greater
- *   than the natural width once all of the said widget's siblings have
+ *   than the natural width once all of the said widget’s siblings have
  *   received their natural widths.
  *   Furthermore, a widget will only ever be allocated a width greater than
  *   its natural width if it was configured to receive extra expand space
@@ -176,36 +227,138 @@ struct _GtkWidget
  *   contextual width to request height for. By implementing this virtual
  *   method it is possible for a #GtkLabel to tell its parent how much height
  *   would be required if the label were to be allocated a said width.
+ * @mnemonic_activate: Activates the @widget if @group_cycling is
+ *   %FALSE, and just grabs the focus if @group_cycling is %TRUE.
+ * @grab_focus: Causes @widget to have the keyboard focus for the
+ *   #GtkWindow it’s inside.
+ * @focus:
+ * @move_focus: Signal emitted when a change of focus is requested
+ * @keynav_failed: Signal emitted if keyboard navigation fails.
+ * @event: The GTK+ main loop will emit three signals for each GDK
+ *   event delivered to a widget: one generic ::event signal, another,
+ *   more specific, signal that matches the type of event delivered
+ *   (e.g. "key-press-event") and finally a generic "event-after"
+ *   signal.
+ * @button_press_event: Signal will be emitted when a button
+ *   (typically from a mouse) is pressed.
+ * @button_release_event: Signal will be emitted when a button
+ *   (typically from a mouse) is released.
+ * @scroll_event: Signal emitted when a button in the 4 to 7 range is
+ *   pressed.
+ * @motion_notify_event: Signal emitted when the pointer moves over
+ *   the widget’s #GdkWindow.
+ * @delete_event: Signal emitted if a user requests that a toplevel
+ *   window is closed.
+ * @destroy_event: Signal is emitted when a #GdkWindow is destroyed.
+ * @key_press_event: Signal emitted when a key is pressed.
+ * @key_release_event: Signal is emitted when a key is released.
+ * @enter_notify_event: Signal event will be emitted when the pointer
+ *   enters the widget’s window.
+ * @leave_notify_event: Will be emitted when the pointer leaves the
+ *   widget’s window.
+ * @configure_event: Signal will be emitted when the size, position or
+ *   stacking of the widget’s window has changed.
+ * @focus_in_event: Signal emitted when the keyboard focus enters the
+ * widget’s window.
+ * @focus_out_event: Signal emitted when the keyboard focus leaves the
+ * widget’s window.
+ * @map_event: Signal emitted when the widget’s window is mapped.
+ * @unmap_event: Signal will be emitted when the widget’s window is
+ *   unmapped.
+ * @property_notify_event: Signal will be emitted when a property on
+ *   the widget’s window has been changed or deleted.
+ * @selection_clear_event: Signal will be emitted when the the
+ *   widget’s window has lost ownership of a selection.
+ * @selection_request_event: Signal will be emitted when another
+ *   client requests ownership of the selection owned by the widget's
+ *   window.
+ * @selection_notify_event:
+ * @proximity_in_event:
+ * @proximity_out_event:
+ * @visibility_notify_event: Signal emitted when the widget’s window is
+ *   obscured or unobscured.
+ * @window_state_event: Signal emitted when the state of the toplevel
+ *   window associated to the widget changes.
+ * @damage_event: Signal emitted when a redirected window belonging to
+ *   widget gets drawn into.
+ * @grab_broken_event: Signal emitted when a pointer or keyboard grab
+ *   on a window belonging to widget gets broken.
+ * @selection_get:
+ * @selection_received:
+ * @drag_begin: Signal emitted on the drag source when a drag is
+ *   started.
+ * @drag_end: Signal emitted on the drag source when a drag is
+ *   finished.
+ * @drag_data_get: Signal emitted on the drag source when the drop
+ *   site requests the data which is dragged.
+ * @drag_data_delete: Signal emitted on the drag source when a drag
+ *   with the action %GDK_ACTION_MOVE is successfully completed.
+ * @drag_leave: Signal emitted on the drop site when the cursor leaves
+ *   the widget.
+ * @drag_motion: signal emitted on the drop site when the user moves
+ *   the cursor over the widget during a drag.
+ * @drag_drop: Signal emitted on the drop site when the user drops the
+ *   data onto the widget.
+ * @drag_data_received: Signal emitted on the drop site when the
+ *   dragged data has been received.
+ * @drag_failed: Signal emitted on the drag source when a drag has
+ *   failed.
+ * @popup_menu: Signal emitted whenever a widget should pop up a
+ *   context menu.
+ * @show_help:
+ * @get_accessible: Returns the accessible object that describes the
+ *   widget to an assistive technology.
+ * @screen_changed: Signal emitted when the screen of a widget has
+ *   changed.
+ * @can_activate_accel: Signal allows applications and derived widgets
+ *   to override the default GtkWidget handling for determining whether
+ *   an accelerator can be activated.
+ * @composited_changed: Signal emitted when the composited status of
+ *   widgets screen changes. See gdk_screen_is_composited().
+ * @query_tooltip: Signal emitted when “has-tooltip” is %TRUE and the
+ *   hover timeout has expired with the cursor hovering “above”
+ *   widget; or emitted when widget got focus in keyboard mode.
+ * @compute_expand: Computes whether a container should give this
+ *   widget extra space when possible.
  * @adjust_size_request: Convert an initial size request from a widget's
  *   #GtkSizeRequestMode virtual method implementations into a size request to
  *   be used by parent containers in laying out the widget.
- *   adjust_size_request adjusts <emphasis>from</emphasis> a child widget's
- *   original request <emphasis>to</emphasis> what a parent container should
+ *   adjust_size_request adjusts from a child widget's
+ *   original request to what a parent container should
  *   use for layout. The @for_size argument will be -1 if the request should
  *   not be for a particular size in the opposing orientation, i.e. if the
  *   request is not height-for-width or width-for-height. If @for_size is
  *   greater than -1, it is the proposed allocation in the opposing
  *   orientation that we need the request for. Implementations of
  *   adjust_size_request should chain up to the default implementation,
- *   which applies #GtkWidget's margin properties and imposes any values
+ *   which applies #GtkWidget’s margin properties and imposes any values
  *   from gtk_widget_set_size_request(). Chaining up should be last,
- *   <emphasis>after</emphasis> your subclass adjusts the request, so
+ *   after your subclass adjusts the request, so
  *   #GtkWidget can apply constraints and add the margin properly.
  * @adjust_size_allocation: Convert an initial size allocation assigned
  *   by a #GtkContainer using gtk_widget_size_allocate(), into an actual
  *   size allocation to be used by the widget. adjust_size_allocation
- *   adjusts <emphasis>to</emphasis> a child widget's actual allocation
- *   <emphasis>from</emphasis> what a parent container computed for the
+ *   adjusts to a child widget’s actual allocation
+ *   from what a parent container computed for the
  *   child. The adjusted allocation must be entirely within the original
  *   allocation. In any custom implementation, chain up to the default
  *   #GtkWidget implementation of this method, which applies the margin
  *   and alignment properties of #GtkWidget. Chain up
- *   <emphasis>before</emphasis> performing your own adjustments so your
+ *   before performing your own adjustments so your
  *   own adjustments remove more allocation after the #GtkWidget base
  *   class has already removed margin and alignment. The natural size
  *   passed in should be adjusted in the same way as the allocated size,
  *   which allows adjustments to perform alignments or other changes
  *   based on natural size.
+ * @style_updated: Signal emitted when the GtkStyleContext of a widget
+ *   is changed.
+ * @touch_event:
+ * @get_preferred_height_and_baseline_for_width:
+ * @adjust_baseline_request:
+ * @adjust_baseline_allocation:
+ * @queue_draw_region: Invalidates the area of widget defined by
+ *   region by calling gdk_window_invalidate_region() on the widget's
+ *   window and all its child windows.
  */
 struct _GtkWidgetClass
 {
@@ -456,6 +609,15 @@ struct _GtkWidgetClass
   void (*_gtk_reserved7) (void);
 };
 
+/**
+ * GtkWidgetAuxInfo:
+ * @width: the widget’s width
+ * @height: the widget’s height
+ * @halign: the widget’s horizontal alignment
+ * @valign: the widget’s horizontal alignment
+ * @margin: the widget’s #GtkBorder margins
+ *
+ */
 struct _GtkWidgetAuxInfo
 {
   gint width;
@@ -784,7 +946,8 @@ GDK_AVAILABLE_IN_ALL
 void                  gtk_widget_set_allocation         (GtkWidget     *widget,
                                                          const GtkAllocation *allocation);
 
-GDK_DEPRECATED_IN_3_0_FOR(gtk_widget_get_preferred_width and gtk_widget_get_preferred_height)
+GDK_DEPRECATED_IN_3_0_FOR(gtk_widget_get_preferred_width & gtk_widget_get_preferred_height)
+
 void                  gtk_widget_get_requisition        (GtkWidget     *widget,
                                                          GtkRequisition *requisition);
 
@@ -852,7 +1015,7 @@ GDK_AVAILABLE_IN_3_10
 gint          gtk_widget_get_scale_factor (GtkWidget *widget);
 GDK_AVAILABLE_IN_ALL
 GdkDisplay *  gtk_widget_get_display     (GtkWidget *widget);
-GDK_AVAILABLE_IN_ALL
+GDK_DEPRECATED_IN_3_12
 GdkWindow *   gtk_widget_get_root_window (GtkWidget *widget);
 GDK_AVAILABLE_IN_ALL
 GtkSettings*  gtk_widget_get_settings    (GtkWidget *widget);
@@ -920,16 +1083,26 @@ GtkAlign gtk_widget_get_valign_with_baseline (GtkWidget *widget);
 GDK_AVAILABLE_IN_ALL
 void     gtk_widget_set_valign        (GtkWidget *widget,
                                        GtkAlign   align);
-GDK_AVAILABLE_IN_ALL
+GDK_DEPRECATED_IN_3_12_FOR(gtk_widget_get_margin_start)
 gint     gtk_widget_get_margin_left   (GtkWidget *widget);
-GDK_AVAILABLE_IN_ALL
+GDK_DEPRECATED_IN_3_12_FOR(gtk_widget_set_margin_start)
 void     gtk_widget_set_margin_left   (GtkWidget *widget,
                                        gint       margin);
-GDK_AVAILABLE_IN_ALL
+GDK_DEPRECATED_IN_3_12_FOR(gtk_widget_get_margin_end)
 gint     gtk_widget_get_margin_right  (GtkWidget *widget);
-GDK_AVAILABLE_IN_ALL
+GDK_DEPRECATED_IN_3_12_FOR(gtk_widget_set_margin_end)
 void     gtk_widget_set_margin_right  (GtkWidget *widget,
                                        gint       margin);
+GDK_AVAILABLE_IN_3_12
+gint     gtk_widget_get_margin_start  (GtkWidget *widget);
+GDK_AVAILABLE_IN_3_12
+void     gtk_widget_set_margin_start  (GtkWidget *widget,
+                                       gint       margin);
+GDK_AVAILABLE_IN_3_12
+gint     gtk_widget_get_margin_end    (GtkWidget *widget);
+GDK_AVAILABLE_IN_3_12
+void     gtk_widget_set_margin_end    (GtkWidget *widget,
+                                       gint       margin);
 GDK_AVAILABLE_IN_ALL
 gint     gtk_widget_get_margin_top    (GtkWidget *widget);
 GDK_AVAILABLE_IN_ALL
